' Title:		iTunes Script Device
' Description:	Permits HouseBot to play Tracks and PlayLists maintained by Apple's iTunes application.
' Author:		Taras Dejneka
' Date:		2006-08-09
' Version:		1.0

Option Explicit
On Error Resume Next	' Comment out this directive if you intend to modify and debug this script.

' Declare public variables
Dim oApp		' Object; IiTunes, iTunes application object.
Dim sAction		' String; Action to be performed by the Script Device.
Dim sArtist		' String; Name of the Artist.
Dim sAlbum		' String; Name of the Album.
Dim sTrack		' String; Name of the Track.
Dim sPlayList	' String; Name of the PlayList.
Dim sEQPreset	' String; Name of the Equalizer Preset.
Dim bEQEnabled	' Boolean; Enable/disable the Equalizer.

' Get iTunes Script Device's properties
sAction		= GetPropertyValue("iTunes.iTunes Action")
sArtist		= GetPropertyValue("iTunes.iTunes Artist")
sAlbum		= GetPropertyValue("iTunes.iTunes Album")
sTrack		= GetPropertyValue("iTunes.iTunes Track")
sPlayList	= GetPropertyValue("iTunes.iTunes PlayList")
sEQPreset	= GetPropertyValue("iTunes.iTunes EQ Preset")
bEQEnabled	= GetPropertyValue("iTunes.iTunes EQ Enabled")

' Create iTunes application object ( will also automatically start the iTunes application)
set oApp = CreateObject("iTunes.Application")

' Identify the Action selected by the user
select case sAction
	' Action: Play Track
	' Play the song identified by the required fields Track and Artist.
	' If supplied, also use the Album field to refine the search for
	' the desired Track.
	case "Play Track"
		if len(sArtist) > 0 and len(sTrack) > 0 then
			call PlayTrack(sArtist, sAlbum, sTrack, bEQEnabled, sEQPreset)
		else
			call ShowInfo(Nothing)
		end if
	
	' Action: Play List
	' Play the PlayList identified by the PlayList field.
	case "Play List"
		if len(sPlayList) > 0 then
			call PlayList(sPlayList, bEQEnabled, sEQPreset)
		else
			call ShowInfo(Nothing)
		end if
	
	' Action: Play / Pause
	' Toggle between pause and play states.
	' If a song is playing -> pause it.
	' If a song is paused, fastforwarding, or rewinding -> play it.
	case "Play / Pause"
		select case oApp.PlayerState
			case 0,1	' Stopped or Playing
				oApp.PlayPause
			case 2,3	' Fast-forwarding or Rewinding
				oApp.Resume
		end select

	' Action: Stop
	' Stop playing the current song.
	case "Stop"
		oApp.Stop
	
	' Action: BackTrack
	' Restart playing the current song.
	case "BackTrack"
		oApp.BackTrack
		call ShowInfo(oApp.CurrentTrack)
	
	' Action: Next
	' Skip the current song and play the next track.
	case "Next"
		oApp.NextTrack
		call ShowInfo(oApp.CurrentTrack)

	' Action: Previous
	' Skip the current song and play the previous track.
	case "Previous"
		oApp.PreviousTrack
		call ShowInfo(oApp.CurrentTrack)
		
	' Action: FastForward
	' Fastforward the current song.
	case "FastForward"
		oApp.FastForward

	' Action: Rewind
	' Rewind the current song.
	case "Rewind"
		oApp.Rewind

	' Action: Info
	' Show information about the current track.
	case "Info"
		call ShowInfo(oApp.CurrentTrack)
	
	' Action: Set EQ
	' Enable/disable the Equalizer.
	case "Set EQ"
		call SetEQ(bEQEnabled,sEQPreset)
	
	' Action: Clear
	' Clear all fields
	case "Clear"
		call ShowInfo(Nothing)
	
	' Action: Unknown
	' Unknown action; erase all properties to indicate something is amiss.
	case else
		call ShowInfo(Nothing)
end select
set oApp = nothing


' Title:		PlayTrack
' Description:	Search for and play the desired Track.
' Details		PlayTrack searches the iTunes Music Library for the song 
'			specified by the fields Track, Artist, and optionally, Album.
'			If found, the song is played.  If not found, the properties
'			(Artist, Track, Album, etc) are erased to indicate the song
'			was not found.
Sub PlayTrack(sArtist, sAlbum, sTrack, bFlag, sName)
	Dim	oTracks	' Object; IITTrackCollection
	Dim	iCnt	' Integer; Counter
	Dim bFound	' Boolean; Flag to indicate the desired Track was located.
	
	' Convert all field text to lowercase characters.
	sArtist = LCase(sArtist)
	sAlbum = LCase(sAlbum)
	sTrack = LCase(sTrack)
	
	bFound = False
	
	' Get all Tracks by the specified Artist.
	set oTracks = oApp.LibraryPlaylist.Search(sArtist,2)
	if oTracks is Nothing then
		' Track was not found; specified Artist doesn't exist.
		call ShowInfo(Nothing)
	else
		with oTracks
			if len(sAlbum) > 1 then ' Search using Track and Album
				for iCnt=1 to .Count
					if instr(1, Lcase(.Item(iCnt).Name), sTrack, 0) and _
						instr(1, Lcase(.Item(iCnt).Album), sAlbum, 0) then
						bFound = True
						exit for
					end if
				next
			else	' Search using Track exclusively
				for iCnt=1 to .Count
					if instr(1, Lcase(.Item(iCnt).Name), sTrack, 0) then
						bFound = True
						exit for
					end if
				next
			end if
			
			if bFound then
				call SetEQ(bFlag, sName)
				.Item(iCnt).Play
				call ShowInfo(.Item(iCnt))
			else
				call ShowInfo(Nothing)
			end if
		end with
	end if
	set oTracks = Nothing
End Sub


' Title:		PlayList
' Description:	Search for and play the desired PlayList.
' Details:		PlayList searches the iTunes Music Library for the PlayList
'			specified by the PlayList field.  If found, the song is played.
'			If not found, the properties (Artist, Track, Album, etc)
'			are erased to indicate the PlayList  was not found.
Sub PlayList(sPlayList, bFlag, sName)
	Dim oPlayList	'Object; IITPlayList
	
	call ShowInfo(Nothing)
	sPlayList = LCase(sPlaylist)
	
	For Each oPlayList In oApp.LibrarySource.Playlists
		with oPlayList
		    If instr(1, Lcase(.Name), sPlayList, 0) Then
				call SetEQ(bFlag, sName)
		        .PlayFirstTrack
				call ShowInfo(oApp.CurrentTrack)
		        Exit For
		    End If
		end with
	Next
	set oPlayList = Nothing
End Sub


' Title:		SetEQ
' Description:	Enable/disable the Equalizer.
' Details:		 If enabled, activate the preset identified  by the EQ Preset property.
Sub SetEQ(bFlag, sName)
	Dim oEQPreset	' Object; IITEQPreset
	
	select case bFlag
		case 0
			oApp.EQEnabled = False
		case 1
			if Len(sName) > 0 then
				set oEQPreset = oApp.EQPresets.ItemByName(sName)
				oApp.CurrentEQPreset = oEQPreset
				oApp.EQEnabled = True
			end if
	end select

End Sub


' Title:		ShowInfo
' Description:	Display information about the current Track.
' Details:		Set the iTunes Script Device's properties using the
'			tags found in the iTunes application's music library
'			(i.e. Artist, Album, Genre, etc) .
Sub ShowInfo(oTrack)
	if oTrack is Nothing then
		SetPropertyValue "iTunes.iTunes~Genre", ""
		SetPropertyValue "iTunes.iTunes Artist", ""
		SetPropertyValue "iTunes.iTunes Album", ""
		SetPropertyValue "iTunes.iTunes~Year", 0
		SetPropertyValue "iTunes.iTunes~TrackNumber", 0
		SetPropertyValue "iTunes.iTunes~Time", ""
		SetPropertyValue "iTunes.iTunes Track", ""
		SetPropertyValue "iTunes.iTunes PlayList", ""
	else
		with oTrack
			SetPropertyValue "iTunes.iTunes~Genre", .Genre
			SetPropertyValue "iTunes.iTunes Artist", .Artist
			SetPropertyValue "iTunes.iTunes Album", .Album
			SetPropertyValue "iTunes.iTunes~Year", .Year
			SetPropertyValue "iTunes.iTunes~TrackNumber", .TrackNumber
			SetPropertyValue "iTunes.iTunes~Time", .Time
			SetPropertyValue "iTunes.iTunes Track", .Name
		end with
		SetPropertyValue "iTunes.iTunes PlayList", oApp.CurrentPlayList.Name
		SetPropertyValue "iTunes.iTunes EQ Preset", oApp.CurrentEQPreset.Name
	end if
End Sub